@@ -49,17 +49,16 @@
 #include "phonenumbers/phonenumberutil.h"
 #include "phonenumbers/regexp_adapter.h"
 #include "phonenumbers/regexp_adapter_icu.h"
+#include "phonenumbers/regexp_cache.h"
 #include "phonenumbers/stringutil.h"
 
 #ifdef I18N_PHONENUMBERS_USE_RE2
 #include "phonenumbers/regexp_adapter_re2.h"
-#endif  // I18N_PHONENUMBERS_USE_RE2_AND_ICU
+#endif  // I18N_PHONENUMBERS_USE_RE2
 
-using std::make_pair;
 using std::map;
 using std::numeric_limits;
 using std::string;
-using std::vector;
 
 namespace i18n {
 namespace phonenumbers {
@@ -118,7 +117,7 @@ bool AllNumberGroupsRemainGrouped(
     const PhoneNumberUtil& util,
     const PhoneNumber& number,
     const string& normalized_candidate,
-    const vector<string>& formatted_number_groups) {
+    const std::vector<string>& formatted_number_groups) {
   size_t from_index = 0;
   if (number.country_code_source() != PhoneNumber::FROM_DEFAULT_COUNTRY) {
     // First skip the country code if the normalized candidate contained it.
@@ -220,10 +219,6 @@ class PhoneNumberMatcherRegExps : public Singleton<PhoneNumberMatcherRegExps> {
   string lead_class_chars_;
   // Same as lead_class_chars_, but enclosed as a character class.
   string lead_class_;
-  // Extra helper strings that form part of pattern_. These are stored
-  // separately since StrCat has a limit of 12 args.
-  string opening_punctuation_;
-  string optional_extn_pattern_;
 
  public:
   // We use two different reg-ex factories here for performance reasons. RE2 is
@@ -232,6 +227,10 @@ class PhoneNumberMatcherRegExps : public Singleton<PhoneNumberMatcherRegExps> {
   scoped_ptr<const AbstractRegExpFactory> regexp_factory_for_pattern_;
   scoped_ptr<const AbstractRegExpFactory> regexp_factory_;
 
+  // A cache for popular reg-exps of leading digits used to match formatting
+  // patterns and the factory used to create it.
+  mutable RegExpCache regexp_cache_;
+
   // Matches strings that look like publication pages. Example:
   // Computing Complete Answers to Queries in the Presence of Limited Access
   // Patterns. Chen Li. VLDB J. 12(3): 211-227 (2003).
@@ -259,7 +258,7 @@ class PhoneNumberMatcherRegExps : public Singleton<PhoneNumberMatcherRegExps> {
   //
   // Note that if there is a match, we will always check any text found up to
   // the first match as well.
-  scoped_ptr<vector<const RegExp*> > inner_matches_;
+  scoped_ptr<std::vector<const RegExp*> > inner_matches_;
   scoped_ptr<const RegExp> capture_up_to_second_number_start_pattern_;
   scoped_ptr<const RegExp> capturing_ascii_digits_pattern_;
   // Compiled reg-ex representing lead_class_;
@@ -288,39 +287,42 @@ class PhoneNumberMatcherRegExps : public Singleton<PhoneNumberMatcherRegExps> {
         digit_sequence_(StrCat("\\p{Nd}", Limit(1, digit_block_limit_))),
         lead_class_chars_(StrCat(opening_parens_, PhoneNumberUtil::kPlusChars)),
         lead_class_(StrCat("[", lead_class_chars_, "]")),
-        opening_punctuation_(StrCat("(?:", lead_class_, punctuation_, ")")),
-        optional_extn_pattern_(StrCat(
-            "(?i)(?:",
-            PhoneNumberUtil::GetInstance()->GetExtnPatternsForMatching(),
-            ")?")),
         regexp_factory_for_pattern_(new ICURegExpFactory()),
 #ifdef I18N_PHONENUMBERS_USE_RE2
         regexp_factory_(new RE2RegExpFactory()),
 #else
         regexp_factory_(new ICURegExpFactory()),
 #endif  // I18N_PHONENUMBERS_USE_RE2
+        // A cache for frequently used country-specific regular expressions. Set
+        // to 32 to cover ~2-3 countries being used for the same doc with ~10
+        // patterns for each country. Some pages will have a lot more countries
+        // in use, but typically fewer numbers for each so expanding the cache
+        // for that use-case won't have a lot of benefit.
+        regexp_cache_(*regexp_factory_, 32),
         pub_pages_(regexp_factory_->CreateRegExp(
             "\\d{1,5}-+\\d{1,5}\\s{0,4}\\(\\d{1,4}")),
         slash_separated_dates_(regexp_factory_->CreateRegExp(
             "(?:(?:[0-3]?\\d/[01]?\\d)|"
             "(?:[01]?\\d/[0-3]?\\d))/(?:[12]\\d)?\\d{2}")),
         time_stamps_(regexp_factory_->CreateRegExp(
-            "[12]\\d{3}[-/]?[01]\\d[-/]?[0-3]\\d [0-2]\\d$")),
+            "[12]\\d{3}[-/]?[01]\\d[-/]?[0-3]\\d +[0-2]\\d$")),
         time_stamps_suffix_(regexp_factory_->CreateRegExp(":[0-5]\\d")),
         matching_brackets_(regexp_factory_->CreateRegExp(
             StrCat(leading_maybe_matched_bracket_, non_parens_, "+",
                    bracket_pairs_, non_parens_, "*"))),
-        inner_matches_(new vector<const RegExp*>()),
+        inner_matches_(new std::vector<const RegExp*>()),
         capture_up_to_second_number_start_pattern_(
             regexp_factory_->CreateRegExp(
                 PhoneNumberUtil::kCaptureUpToSecondNumberStart)),
         capturing_ascii_digits_pattern_(
             regexp_factory_->CreateRegExp("(\\d+)")),
         lead_class_pattern_(regexp_factory_->CreateRegExp(lead_class_)),
-        pattern_(regexp_factory_for_pattern_->CreateRegExp(
-            StrCat("(", opening_punctuation_, lead_limit_,
-                   digit_sequence_, "(?:", punctuation_, digit_sequence_, ")",
-                   block_limit_, optional_extn_pattern_, ")"))) {
+        pattern_(regexp_factory_for_pattern_->CreateRegExp(StrCat(
+            "((?:", lead_class_, punctuation_, ")", lead_limit_,
+            digit_sequence_, "(?:", punctuation_, digit_sequence_, ")",
+            block_limit_, "(?i)(?:",
+            PhoneNumberUtil::GetInstance()->GetExtnPatternsForMatching(),
+            ")?)"))) {
     inner_matches_->push_back(
         // Breaks on the slash - e.g. "651-234-2345/332-445-1234"
         regexp_factory_->CreateRegExp("/+(.*)"));
@@ -340,7 +342,7 @@ class PhoneNumberMatcherRegExps : public Singleton<PhoneNumberMatcherRegExps> {
         // break two numbers without spaces, and we haven't seen many instances
         // of it used within a number.
         regexp_factory_->CreateRegExp(
-            "[\xE2\x80\x92-\x2D\xE2\x80\x95\xEF\xBC\x8D]" /* "‒-―－" */
+            "[\xE2\x80\x92-\xE2\x80\x95\xEF\xBC\x8D]" /* "‒-―－" */
             "\\p{Z}*(.+)"));
     inner_matches_->push_back(
         // Breaks on a full stop - e.g. "12345. 332-445-1234 is my number."
@@ -372,7 +374,7 @@ class AlternateFormats : public Singleton<AlternateFormats> {
          it != format_data_.metadata().end();
          ++it) {
       calling_code_to_alternate_formats_map_.insert(
-          make_pair(it->country_code(), &*it));
+          std::make_pair(it->country_code(), &*it));
     }
   }
 
@@ -490,35 +492,6 @@ bool PhoneNumberMatcher::ParseAndVerify(const string& candidate, int offset,
     return false;
   }
 
-
-  // Check Israel * numbers: these are a special case in that they are
-  // four-digit numbers that our library supports, but they can only be dialled
-  // with a leading *. Since we don't actually store or detect the * in our
-  // phone number library, this means in practice we detect most four digit
-  // numbers as being valid for Israel. We are considering moving these numbers
-  // to ShortNumberInfo instead, in which case this problem would go away, but
-  // in the meantime we want to restrict the false matches so we only allow
-  // these numbers if they are preceded by a star. We enforce this for all
-  // leniency levels even though these numbers are technically accepted by
-  // isPossibleNumber and isValidNumber since we consider it to be a deficiency
-  // in those methods that they accept these numbers without the *.
-  // TODO: Remove this or make it significantly less hacky once
-  // we've decided how to handle these short codes going forward in
-  // ShortNumberInfo. We could use the formatting rules for instance, but that
-  // would be slower.
-  string region_code;
-  phone_util_.GetRegionCodeForCountryCode(number.country_code(), &region_code);
-  if (region_code == "IL") {
-    string national_number;
-    phone_util_.GetNationalSignificantNumber(number, &national_number);
-    if (national_number.length() == 4 &&
-        // Just check the previous char, since * is an ASCII character.
-        (offset == 0 || (offset > 0 && text_[offset - 1] != '*'))) {
-      // No match.
-      return false;
-    }
-  }
-
   if (VerifyAccordingToLeniency(leniency_, number, candidate)) {
     match->set_start(offset);
     match->set_raw_string(candidate);
@@ -558,7 +531,7 @@ bool PhoneNumberMatcher::VerifyAccordingToLeniency(
         return false;
       }
       ResultCallback4<bool, const PhoneNumberUtil&, const PhoneNumber&,
-                      const string&, const vector<string>&>* callback =
+                      const string&, const std::vector<string>&>* callback =
           NewPermanentCallback(&AllNumberGroupsRemainGrouped);
       bool is_valid = CheckNumberGroupingIsValid(number, candidate, callback);
       delete(callback);
@@ -573,7 +546,7 @@ bool PhoneNumberMatcher::VerifyAccordingToLeniency(
         return false;
       }
       ResultCallback4<bool, const PhoneNumberUtil&, const PhoneNumber&,
-                      const string&, const vector<string>&>* callback =
+                      const string&, const std::vector<string>&>* callback =
           NewPermanentCallback(
               this, &PhoneNumberMatcher::AllNumberGroupsAreExactlyPresent);
       bool is_valid = CheckNumberGroupingIsValid(number, candidate, callback);
@@ -590,7 +563,7 @@ bool PhoneNumberMatcher::VerifyAccordingToLeniency(
 bool PhoneNumberMatcher::ExtractInnerMatch(const string& candidate, int offset,
                                            PhoneNumberMatch* match) {
   DCHECK(match);
-  for (vector<const RegExp*>::const_iterator regex =
+  for (std::vector<const RegExp*>::const_iterator regex =
            reg_exps_->inner_matches_->begin();
            regex != reg_exps_->inner_matches_->end(); regex++) {
     scoped_ptr<RegExpInput> candidate_input(
@@ -705,30 +678,42 @@ bool PhoneNumberMatcher::CheckNumberGroupingIsValid(
     const PhoneNumber& phone_number,
     const string& candidate,
     ResultCallback4<bool, const PhoneNumberUtil&, const PhoneNumber&,
-                    const string&, const vector<string>&>* checker) const {
+                    const string&, const std::vector<string>&>* checker) const {
   DCHECK(checker);
-  // TODO: Evaluate how this works for other locales (testing has been limited
-  // to NANPA regions) and optimise if necessary.
   string normalized_candidate =
       NormalizeUTF8::NormalizeDecimalDigits(candidate);
-  vector<string> formatted_number_groups;
-  GetNationalNumberGroups(phone_number, NULL,  // Use default formatting pattern
-                          &formatted_number_groups);
+  std::vector<string> formatted_number_groups;
+  GetNationalNumberGroups(phone_number, &formatted_number_groups);
   if (checker->Run(phone_util_, phone_number, normalized_candidate,
                    formatted_number_groups)) {
     return true;
   }
-  // If this didn't pass, see if there are any alternate formats, and try them
-  // instead.
+  // If this didn't pass, see if there are any alternate formats that match, and
+  // try them instead.
   const PhoneMetadata* alternate_formats =
     alternate_formats_->GetAlternateFormatsForCountry(
         phone_number.country_code());
   if (alternate_formats) {
+    string national_significant_number;
+    phone_util_.GetNationalSignificantNumber(phone_number,
+                                             &national_significant_number);
     for (RepeatedPtrField<NumberFormat>::const_iterator it =
              alternate_formats->number_format().begin();
          it != alternate_formats->number_format().end(); ++it) {
+      if (it->leading_digits_pattern_size() > 0) {
+        std::unique_ptr<RegExpInput> nsn_input(
+            reg_exps_->regexp_factory_->CreateInput(
+                national_significant_number));
+        // There is only one leading digits pattern for alternate formats.
+        if (!reg_exps_->regexp_cache_.GetRegExp(
+                it->leading_digits_pattern(0)).Consume(nsn_input.get())) {
+          // Leading digits don't match; try another one.
+          continue;
+        }
+      }
       formatted_number_groups.clear();
-      GetNationalNumberGroups(phone_number, &*it, &formatted_number_groups);
+      GetNationalNumberGroupsForPattern(phone_number, &*it,
+                                        &formatted_number_groups);
       if (checker->Run(phone_util_, phone_number, normalized_candidate,
                        formatted_number_groups)) {
         return true;
@@ -738,40 +723,40 @@ bool PhoneNumberMatcher::CheckNumberGroupingIsValid(
   return false;
 }
 
-// Helper method to get the national-number part of a number, formatted without
-// any national prefix, and return it as a set of digit blocks that would be
-// formatted together.
 void PhoneNumberMatcher::GetNationalNumberGroups(
     const PhoneNumber& number,
-    const NumberFormat* formatting_pattern,
-    vector<string>* digit_blocks) const {
+    std::vector<string>* digit_blocks) const {
   string rfc3966_format;
-  if (!formatting_pattern) {
-    // This will be in the format +CC-DG;ext=EXT where DG represents groups of
-    // digits.
-    phone_util_.Format(number, PhoneNumberUtil::RFC3966, &rfc3966_format);
-    // We remove the extension part from the formatted string before splitting
-    // it into different groups.
-    size_t end_index = rfc3966_format.find(';');
-    if (end_index == string::npos) {
-      end_index = rfc3966_format.length();
-    }
-    // The country-code will have a '-' following it.
-    size_t start_index = rfc3966_format.find('-') + 1;
-    SplitStringUsing(rfc3966_format.substr(start_index,
-                                           end_index - start_index),
-                     "-", digit_blocks);
-  } else {
-    // We format the NSN only, and split that according to the separator.
-    string national_significant_number;
-    phone_util_.GetNationalSignificantNumber(number,
-                                             &national_significant_number);
-    phone_util_.FormatNsnUsingPattern(national_significant_number,
-                                      *formatting_pattern,
-                                      PhoneNumberUtil::RFC3966,
-                                      &rfc3966_format);
-    SplitStringUsing(rfc3966_format, "-", digit_blocks);
+  // This will be in the format +CC-DG1-DG2-DGX;ext=EXT where DG1..DGX
+  // represents groups of digits.
+  phone_util_.Format(number, PhoneNumberUtil::RFC3966, &rfc3966_format);
+  // We remove the extension part from the formatted string before splitting
+  // it into different groups.
+  size_t end_index = rfc3966_format.find(';');
+  if (end_index == string::npos) {
+    end_index = rfc3966_format.length();
   }
+  // The country-code will have a '-' following it.
+  size_t start_index = rfc3966_format.find('-') + 1;
+  SplitStringUsing(rfc3966_format.substr(start_index,
+                                         end_index - start_index),
+                   "-", digit_blocks);
+}
+
+void PhoneNumberMatcher::GetNationalNumberGroupsForPattern(
+    const PhoneNumber& number,
+    const NumberFormat* formatting_pattern,
+    std::vector<string>* digit_blocks) const {
+  string rfc3966_format;
+  // We format the NSN only, and split that according to the separator.
+  string national_significant_number;
+  phone_util_.GetNationalSignificantNumber(number,
+                                           &national_significant_number);
+  phone_util_.FormatNsnUsingPattern(national_significant_number,
+                                    *formatting_pattern,
+                                    PhoneNumberUtil::RFC3966,
+                                    &rfc3966_format);
+  SplitStringUsing(rfc3966_format, "-", digit_blocks);
 }
 
 bool PhoneNumberMatcher::IsNationalPrefixPresentIfRequired(
@@ -825,10 +810,10 @@ bool PhoneNumberMatcher::AllNumberGroupsAreExactlyPresent(
     const PhoneNumberUtil& util,
     const PhoneNumber& phone_number,
     const string& normalized_candidate,
-    const vector<string>& formatted_number_groups) const {
+    const std::vector<string>& formatted_number_groups) const {
   const scoped_ptr<RegExpInput> candidate_number(
       reg_exps_->regexp_factory_->CreateInput(normalized_candidate));
-  vector<string> candidate_groups;
+  std::vector<string> candidate_groups;
   string digit_block;
   while (reg_exps_->capturing_ascii_digits_pattern_->FindAndConsume(
              candidate_number.get(),
